function [myTestcase] = Testcase13_CDR_2Dc_v4()
    
    %% initialise testcase
    myTestcase          = Testcase('CDR\_2Dc\_v4');
    sSaveName           = 'Testcase13_CDR_2Dc_v4';
    
    %% generate problem instance
    [A_fun,B,X0]        = GenerateLinearSystem();
    [terminate]         = GenerateTermination();
    [param]             = GenerateDefaultParam();
    
    %% 1. System
    myTestcase.pushSystem( A_fun, B(:,1), X0(:,1) );
    myTestcase.setTerminate( terminate );
    myTestcase.setDefaultParam( param );
    % % make solvers for this system
    cIDRstabbio       	= makeSolver(myTestcase,'IDRstabbiortho');
    cIDRstabbio.recyclingparam.tolabs2      = 1e-3;
    cGMstab             = makeSolver(myTestcase,'GMstab');
    cGMstab.recyclingparam.tolabs2         	= 1e-3;
    
    myTestcase.pushSolver(cIDRstabbio);
    myTestcase.pushSolver(cGMstab);
    
    %% 2. System
    myTestcase.pushSystem( A_fun, B(:,2), X0(:,2) );
    myTestcase.setTerminate( terminate );
    myTestcase.setDefaultParam( param );
    % % make solvers for this system
    cIDRstabbio       	= makeSolver(myTestcase,'IDRstabbiortho');
    cIDRstabbio.recyclingparam.systemindex  = 1;
    cIDRstabbio.recyclingparam.solverindex  = 1;
    cGMstab             = makeSolver(myTestcase,'GMstab');
    cGMstab.recyclingparam.systemindex      = 1;
    cGMstab.recyclingparam.solverindex      = 2;
    
    myTestcase.pushSolver(cIDRstabbio);
    myTestcase.pushSolver(cGMstab);
    
    %% Run testcase
    myTestcase.Run();
    
    %% Save plot data
    plotData            = myTestcase.getPlotData();
    save(['TestResults/',sSaveName,'_data.mat'],'plotData');
    window_matvec     	= struct('xL',0,'xR',2500,'dx',250,'yL',-12,'yR',4,'dy',2);
    window_runtime     	= struct('xL',0,'xR', 400,'dx', 50,'yL',-12,'yR',4,'dy',2);
    plot_trueres_over_matvec( plotData,window_matvec, ['TestResults/',sSaveName,'_trueres_matvec.pdf']);
    plot_iterres_over_matvec( plotData,window_matvec, ['TestResults/',sSaveName,'_iterres_matvec.pdf']);
    plot_trueres_over_runtime(plotData,window_runtime,['TestResults/',sSaveName,'_trueres_runtime.pdf']);
    plot_iterres_over_runtime(plotData,window_runtime,['TestResults/',sSaveName,'_iterres_runtime.pdf']);
end

function [A_fun,B,X0]  	= GenerateLinearSystem()
    alp         = 0;
    bet         = 1000;
    n           = 350;
    [A,u,b] = example_IDRstab_CDR_2D_alp_bet(alp,bet,n);
    b1       	= b;
    b1       	= 1/norm(b1) * b1;
    b2        	= u;
    b2         	= 1/norm(b2) * b2;
    % generate outputs
    A_fun     	= @(v) A*v;
    B           = [b1,b2];
    X0          = 0*B;
end

function [terminate]    = GenerateTermination()
 	terminate.tolabs            = 1e-10;
    terminate.maxmatvec         = 2500;
    terminate.maxruntime        = 400;
end

function [param]        = GenerateDefaultParam()
    param.s  	= 4;
    param.L   	= 2;
end