classdef Testcase < handle
    
    properties(SetAccess=private, GetAccess=public)
        sName           % name of the testcase
        listSystems   	% list of equation systems
        indexSystems    % list index of lSystems
        listCalls       % contains all the solver calls
    end %properties
    
    methods(Access=private)
        
        function [out]      = CallSolver(obj,in,sSolvername)                    %#ok<MANU>
            switch sSolvername
                case 'GMRES'
                    out                                 = Solver_GMRES(in);
                %end
                case 'IDRref'
                    out                                 = Solver_IDRs_ref(in);
                %end
                case 'IDRbiortho'
                    out                                 = Solver_IDRs_biortho(in);
                %end
                case 'IDRobio'
                    out                                 = Solver_IDRs_obio(in);
                %end
                case 'IDRstabref'
                    out                                 = Solver_IDRsStab_ref(in);
                %end
                case 'IDRstabbiortho'
                    out                                 = Solver_IDRsStab_biortho(in);
                %end
                case 'IDRstabobio'
                    out                                 = Solver_IDRsStab_obio(in);
                %end
                case 'GMstab'
                    out                                 = Solver_GMsStab(in);
                %end
            end
        end
        
        function []         = make_listCalls(obj)
            listCalls                           = [];                                                               %#ok<PROP>
            indexCalls                          = 0;
            N_listSystems                       = length( obj.listSystems );
            for i=1:N_listSystems
                N_listSolvers                       = length( obj.listSystems(i).listSolvers );
                for j=1:N_listSolvers
                    indexCalls                          = indexCalls + 1;
                    listCalls(indexCalls).cSolver     	= obj.listSystems(i).listSolvers(j);                        %#ok<PROP,AGROW>
                    listCalls(indexCalls).system_number	= i;                                                     	%#ok<PROP,AGROW>
                    listCalls(indexCalls).in           	= obj.listSystems(i).listSolvers(j).in;                  	%#ok<PROP,AGROW>
                    listCalls(indexCalls).out         	= obj.listSystems(i).listSolvers(j).out;                 	%#ok<PROP,AGROW>
                end
            end
            obj.listCalls                       = listCalls;                                                        %#ok<PROP>
        end
        
        function cSolver 	= FillcSolver(obj,cSolver)
            defaultParam                        = obj.listSystems(obj.indexSystems).param;
            if( ~isfield(cSolver,'param') )
                cSolver.param                       = defaultParam;
            else
                % check required params
                if( ~isfield(cSolver.param,'s') )
                    cSolver.param.s                     = defaultParam.s;
                end
                if( ~isfield(cSolver.param,'L') )
                    cSolver.param.L                     = defaultParam.L;
                end
            end
        end
        
    end
    
    methods(Access=public)  % methods for instantiation
        
        function [obj] = Testcase(sName)
            obj.sName                           = sName;
            obj.indexSystems                    = 0;
        end
        
        function [] = pushSystem(obj,A_fun,b,x0)
            obj.indexSystems                    = obj.indexSystems + 1;
            i                                   = obj.indexSystems;
            obj.listSystems(i).A_fun            = A_fun;
            obj.listSystems(i).b                = b;
            obj.listSystems(i).x0               = x0;
            % initialise solver list
            obj.listSystems(i).indexSolvers   	= 0;
        end
        
        function [] = setTerminate(obj,terminate)
            i                                   = obj.indexSystems;
            obj.listSystems(i).terminate        = terminate;
        end
        
        function [] = setDefaultParam(obj,param)
            i                                   = obj.indexSystems;
            obj.listSystems(i).param            = param;
        end
        
        function [] = pushSolver(obj,cSolver)
            % fill missing parameters of this solver with defaults
            cSolver                             = obj.FillcSolver(cSolver);
            % push cSolver into the array
            i                                   = obj.indexSystems;
            obj.listSystems(i).indexSolvers     = obj.listSystems(i).indexSolvers + 1;
            j                                   = obj.listSystems(i).indexSolvers;
            % This is what should stand there: obj.listSystems(i).listSolvers(j)  	= cSolver;
            % The following is what one must write:
            obj.listSystems(i).listSolvers(j).sName     = cSolver.sName;
            obj.listSystems(i).listSolvers(j).param     = cSolver.param;
            obj.listSystems(i).listSolvers(j).plotparam = cSolver.plotparam;
            if( isfield(cSolver,'recyclingparam') )
                obj.listSystems(i).listSolvers(j).recyclingparam    = cSolver.recyclingparam;
            end
        end
        
        function [] = Run(obj)
            N_systems                                       = length( obj.listSystems );
            for i=1:N_systems
                N_solvers                                       = length( obj.listSystems(i).listSolvers );
                % generate in structure for this system
                sys_in.A_fun                                    = obj.listSystems(i).A_fun;
                sys_in.b                                      	= obj.listSystems(i).b;
                sys_in.x0                                    	= obj.listSystems(i).x0;
                sys_in.terminate                                = obj.listSystems(i).terminate;
                for j=1:N_solvers
                    SolverName                                     	= obj.listSystems(i).listSolvers(j).sName;
                    % generate in-structure for this solver on this system
                    in                                              = sys_in;
                    in.param                                        = obj.listSystems(i).listSolvers(j).param;
                    if( isfield(obj.listSystems(i).listSolvers(j),'recyclingparam') )
                        
                        if(        isfield(obj.listSystems(i).listSolvers(j).recyclingparam,'systemindex')       	...
                                && isfield(obj.listSystems(i).listSolvers(j).recyclingparam,'solverindex')          ...
                                && ~isnan( obj.listSystems(i).listSolvers(j).recyclingparam.systemindex  )          ...
                                && ~isnan( obj.listSystems(i).listSolvers(j).recyclingparam.solverindex  )          )
                            
                            if( isfield(obj.listSystems(i).listSolvers(j).recyclingparam,'tolabs2') )
                                fprintf('WARNING: system %i, solver %i has both hU and tolabs2.\n',i,j);
                            end
                            % in this case Mstab is called
                            i_rec                                       = obj.listSystems(i).listSolvers(j).recyclingparam.systemindex;
                            j_rec                                       = obj.listSystems(i).listSolvers(j).recyclingparam.solverindex;
                            if( isfield(obj.listSystems(i_rec).listSolvers(j_rec).out,'recyclingparam') && isfield(obj.listSystems(i_rec).listSolvers(j_rec).out.recyclingparam,'hU') )
                                in.recyclingparam.hU                        = obj.listSystems(i_rec).listSolvers(j_rec).out.recyclingparam.hU;
                            else
                                fprintf('WARNING: {system %i, solver %i} wants to access hU \n         from {system %i, solver %i}, which does not exist.\n',i,j,i_rec,j_rec);
                            end
                        else
                            % in this case IDRstab is called and prepares data for Mstab
                            if( isfield(obj.listSystems(i).listSolvers(j).recyclingparam,'tolabs2') )
                                in.recyclingparam.tolabs2               	= obj.listSystems(i).listSolvers(j).recyclingparam.tolabs2;
                            end
                        end
                    end
                    obj.listSystems(i).listSolvers(j).in            = in;
                    % generate out-structure for this solver on this system
                    obj.listSystems(i).listSolvers(j).out           = obj.CallSolver(in,SolverName);
                end
            end
        end
        
        function [plotData] = getPlotData(obj)
            plotData.NameTestcase               = obj.sName;
            obj.make_listCalls();
            plotData.listCalls                  = obj.listCalls;
        end
        
    end %methods
    
end %class Testcase